#--------------------------------------------------------------------------------
# FILE:       QueryCustomerOrg.ps1
#
# PROJECT:    Exchange 2010 /Hosting mode to Exchange 2010 SP2 Migration
#
# PURPOSE:    Sample script that queries Exchange and Active Directory in the 
#             Exchange 2010 /Hosting mode forest for a customer's User, Group,
#	      Contact, and SMTP domain information, then creates an XML output 
#	      file containing the query results.
#
#
# THIS CODE AND INFORMATION IS PROVIDED TO YOU FOR YOUR REFERENTIAL PURPOSES 
# ONLY, AND IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EITHER 
# EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE, AND MAY NOT 
# BE REDISTRIBUTED IN ANY MANNER.
#
# Copyright (C) 2012 Microsoft Corporation. All rights reserved.
#--------------------------------------------------------------------------------

### Modify these variables with values appropriate to your environment ###

$PreferredDC = "ad01.hostingmode.com"
$DomainID = "hostingmode"
$DomainSuffix = "com"
$HostingOU = "Microsoft Exchange Hosted Organizations"

$BaseSearchPath = "OU=$HostingOU,DC=$DomainID,DC=$DomainSuffix"

################### You should not need to edit any variables below this line ###################

if ($args.length -ne 1)
{
	Write-Host "You must provide the name of the customer OU as an argument." -foregroundcolor yellow
	Write-Host "if there are spaces in the OU name, surround it with quotes." -foregroundcolor yellow
	Write-Host "Exiting script." -foregroundcolor yellow
	exit
}
$CustomerID = $args[0]

$OutputFile = "C:\Scripts\$CustomerID.xml"

# import the migration xml schema
$error.clear()
$oldErrorCount = $error.Count
$xml = [xml](get-content "C:\Scripts\MigrationSchema.xml") 

if ($error.Count -ne $oldErrorCount)
{
	[string]$problem = $error[0].Exception
	if ($problem.contains("because it does not exist"))
	{
		write-host "Unable to find C:\Scripts\MigrationSchema.xml" -foregroundcolor yellow
		write-host "this file must be present in the C:\Scripts directory in order for the script to run." -foregroundcolor yellow
		write-host "Exiting script." -foregroundcolor yellow
		exit
	}
	Else
	{
		write-host $error[0].Exception -foregroundcolor red
		exit
	}
}

# save customer name to XML output
$xml.response.data.customerOU = $CustomerID

#Create a dynamic array to hold users
$arrUsersToMigrate = @()

#Create a dynamic array to hold tenant's accepted SMTP domains
$arrSMTPDomains = @()

#Array of system mailboxes to be skipped
$arrSystemMailboxes = "DefaultMailboxPlan","DiscoverySearchMailbox","SystemMailbox"

$SMTPDomainsFound = 0
$UsersFound = 0
$MailboxesFound = 0
$GroupsFound = 0
$MailGroupsFound = 0
$ContactsFound = 0
$MailContactsFound = 0

####################################
#Function to add an XML child node
####################################
function AddChildXML ($XMLParentNode,$ChildNode)
{
	[void]$XMLParentNode.AppendChild($ChildNode)
}

####################################
#Function to perform LDAP query 
####################################

function SearchLDAP ($searchRoot,$filter)
{
	$searcher = New-Object System.DirectoryServices.DirectorySearcher
	$searcher.SearchRoot = $searchRoot
	$searcher.PageSize = 1000
	$searcher.Filter = $filter
	$searcher.SearchScope = "Onelevel" # Set to Base, Onelevel or Subtree
	$searchResults = $searcher.FindAll() # Set to FindOne or FindAll

	# dispose of the AD searcher object in order to clean house
	$searcher.Dispose()

        if ($searchResults -eq $null -or $searchResults.count -eq 0)
        {
            return $null
        }
	else
	{
		return $searchResults
	}
}
#######################################################################
# Retreive the tenant's accepted SMTP domains, save results to XML file

Write-Host "Retrieving tenant's accepted SMTP domains..." -foregroundcolor cyan

$arrSMTPDomains = Get-AcceptedDomain -Organization $CustomerID

foreach ($domain in $arrSMTPDomains)
{
	$SMTPDomainsFound += 1
	[string]$DomainName = $domain.DomainName.SmtpDomain
	[string]$DomainFriendlyName = $domain.Name
	[string]$DomainType = $domain.DomainType
	
	Write-Host "domain: $DomainName"

	# Save the accepted domain  info into the XML dom

	# set the XML parent node
	$XMLParentNode = $xml.response.data.SMTPDomains

	# select the node to clone
	$XMLSMTPNode = @($XMLParentNode.SMTPDomain)[0]

	# clone the node
	$NewSMTPDomain = $XMLSMTPNode.Clone()

	# Set the values of the new node
	$NewSMTPDomain.domainFriendlyName = $DomainFriendlyName
	$NewSMTPDomain.domainName = $DomainName
	$NewSMTPDomain.domainType = $DomainType

	# call the function to add the node
	AddChildXML $XMLParentNode $NewSMTPDomain
}

##########################################################################################################
#Search the customer OU for all users, save results to XML file
$filter = "(&(objectcategory=Person)(objectClass=User))"
$searchRoot = "LDAP://OU=$CustomerID,$BaseSearchPath"
Write-Host "Searching for Users in Customer OU: " $searchRoot -foregroundcolor cyan

#call the SearchLDAP function
$searchResults = SearchLDAP $searchRoot $filter

foreach ($objResult in $searchResults)
{
	# clear out the dynamic array for user's proxy addresses
	$arrUserProxyAddrs = @()
	
	# clear out the dynamic array for the user's 'member of' values (if any)
	$arrUserMemberOf = @()

	# list each property you want to retrieve.  attribute must be specified in all lower case!
	$objItem = $objResult.Properties
	[string]$UserCN = $objItem.cn
	[string]$UserDN = $objItem.distinguishedname
	[string]$UserDisplayName = $objItem.displayname
	[string]$UserUPN = $objItem.userprincipalname
	[string]$UserSAM = $objItem.samaccountname
	[string]$UserName = $objItem.name
	[string]$UserAlias = $objItem.mailnickname
	[string]$UserLED = $objItem.legacyexchangedn
	[string]$UserMailboxPlan = $objItem.msexchparentplanlink
	[string]$UserOWAPolicy = $objItem.msexchowapolicy
	$arrUserProxyAddrs = $objItem.proxyaddresses
	$arrUserMemberOf = $objItem.memberof

	Write-Host $UserName

	# filter out system mailboxes/accounts - we don't want to migrate them
	if ($UserAlias -gt "" -and ($UserAlias.contains("SystemMailbox") -or $UserAlias.contains("DiscoverySearchMailbox") -or $UserAlias.contains("DefaultMailboxPlan")))
	{
		# skip system mailboxes.
		Write-Host "skipping system mailbox: $UserAlias" -foregroundcolor yellow
		continue
	}
	Else
	{
		
		### Save the user account info into the XML dom
				
		# set the XML parent node
		$XMLParentNode = $xml.response.data.users

		# select the node to clone
		$XMLUserNode = @($XMLParentNode.user)[0]

		# clone the node
		$NewUser = $XMLUserNode.Clone()

		# Set the values of the new node
		$NewUser.cn = $UserCN
		if ($UserCN -gt "")
		{
			$UsersFound += 1
		}
		$NewUser.Name = $UserName
		$NewUser.distinguishedName = $UserDN
		$NewUser.displayName = $UserDisplayName
		$NewUser.sAMAccountName = $UserSAM
		$NewUser.userPrincipalName = $UserUPN
		$NewUser.legacyExchangeDN = $UserLED
		$NewUser.mailboxPlan = $UserMailboxPlan
		$NewUser.owaPolicy = $UserOWAPolicy
		if ($UserAlias -gt "")
		{
			$MailboxesFound += 1
			$NewUser.mailEnabled = "TRUE"
			$NewUser.mailNickname = $UserAlias
		}
		Else
		{
			$NewUser.mailEnabled = "FALSE"
		}
		if ($arrUserProxyAddrs -gt "")
		{
			foreach ($proxyAddr in $arrUserProxyAddrs)
			{
				if ($NewUser.proxyAddresses.value -eq "")
				{
					# there is an empty <value> node we can use
					$NewUser.proxyAddresses.value = $proxyAddr
				}
				else
				{
					# create a new <value> node
					$NewValue = $xml.CreateElement("value")
					$NewValue.set_InnerXML("$proxyAddr")
					[void]$NewUser.proxyAddresses.AppendChild($NewValue)
				}
			}
		}

		if ($arrUserMemberOf -gt "")
		{
			foreach ($groupDN in $arrUserMemberOf)
			{
				if ($NewUser.memberOf.value -eq "")
				{
					# there is an empty <value> node we can use
					$NewUser.memberOf.value = $groupDN
				}
				else
				{
					# create a new <value> node
					$NewUserValue = $xml.CreateElement("value")
					$NewUserValue.set_InnerXML("$groupDN")
					[void]$NewUser.memberOf.AppendChild($NewUserValue)
				}
			}
		}

		# call the function to add the node
		AddChildXML $XMLParentNode $NewUser
	}
}
Write-Host "*******************************"

##########################################################################################################
#Search the customer OU for all groups, save results to xml file

$filter = "(&(objectcategory=Group)(objectClass=Group))"
$searchRoot = "LDAP://OU=$CustomerID,$BaseSearchPath"
Write-Host "Searching for Groups in Customer OU: " $searchRoot -foregroundcolor cyan

#call the SearchLDAP function
$searchResults = SearchLDAP $searchRoot $filter

foreach ($objResult in $searchResults)
{
	# clear out the dynamic array for group proxy addresses
	$arrGroupProxyAddrs = @()
	
	# clear out the dynamic array for the group's 'member of' values (if any)
	$arrGroupMemberOf = @()

	# list each property you want to retrieve.  attribute must be specified in all lower case!
	$objItem = $objResult.Properties
	[string]$GroupCN = $objItem.cn
	if ($GroupCN -gt "")
	{
		$GroupsFound += 1
	}
	[string]$GroupDN = $objItem.distinguishedname
	[string]$GroupDisplayName = $objItem.displayname
	[string]$GroupSAM = $objItem.samaccountname
	[string]$GroupName = $objItem.name
	[string]$GroupAlias = $objItem.mailnickname
	[string]$GroupLED = $objItem.legacyexchangedn
	[string]$GroupType = $objItem.grouptype
	$arrGroupProxyAddrs = $objItem.proxyaddresses
	$arrGroupMemberOf = $objItem.memberof

	Write-Host $GroupName

	# filter out system groups from Exchange 2010 SP1 /Hosting - we don't want to migrate them
	if ($GroupName -eq "Discovery Management" -or $GroupName -eq "Help Desk" -or $GroupName -eq "Organization Management" -or $GroupName -eq "Recipient Management" -or $GroupName -eq "Records Management" -or $GroupName -eq "View-Only Organization Management")
	{
		# skip system mailboxes.
		Write-Host "skipping system group: $GroupName" -foregroundcolor yellow
		continue
	}
	Else
	{
		### Save the group account info into the XML dom
		
		# set the XML parent node
		$XMLParentNode = $xml.response.data.groups

		# select the node to clone
		$XMLGroupNode = @($XMLParentNode.group)[0]

		# clone the node
		$NewGroup = $XMLGroupNode.Clone()

		# Set the values of the new node
		$NewGroup.cn = $GroupCN
		$NewGroup.distinguishedName = $GroupDN
		$NewGroup.displayName = $GroupDisplayName
		$NewGroup.sAMAccountName = $GroupSAM
		$NewGroup.legacyExchangeDN = $GroupLED
		$NewGroup.Name = $GroupName
		if ($GroupType -eq "2")
		{
			#found a global distribution group
			$NewGroup.groupScope = "global"
			$NewGroup.groupType = "distribution"
		}
		if ($GroupType -eq "8")
		{
			#found a universal distribution group
			$NewGroup.groupScope = "universal"
			$NewGroup.groupType = "distribution"
		}
		if ($GroupType -eq "-2147483640")
		{
			#found a universal security group
			$NewGroup.groupScope = "universal"
			$NewGroup.groupType = "security"
		}
		if ($GroupType -eq "-2147483646")
		{
			#found a global security group
			$NewGroup.groupScope = "global"
			$NewGroup.groupType = "security"
		}
		if ($GroupAlias -gt "")
		{
			$MailGroupsFound += 1
			$NewGroup.mailEnabled = "TRUE"
			$NewGroup.mailNickname = $GroupAlias
		}
		Else
		{
			$NewGroup.mailEnabled = "FALSE"
		}

		if ($arrGroupProxyAddrs -gt "")
		{
			foreach ($proxyAddr in $arrGroupProxyAddrs)
			{
				if ($NewGroup.proxyAddresses.value -eq "")
				{
					# there is an empty <value> node we can use
					$NewGroup.proxyAddresses.value = $proxyAddr
				}
				else
				{
					# create a new <value> node
					$NewValue = $xml.CreateElement("value")
					$NewValue.set_InnerXML("$proxyAddr")
					[void]$NewGroup.proxyAddresses.AppendChild($NewValue)
				}
			}
			
		}
		if ($arrGroupMemberOf -gt "")
		{
			foreach ($groupDN in $arrGroupMemberOf)
			{
				if ($NewGroup.memberOf.value -eq "")
				{
					# there is an empty <value> node we can use
					$NewGroup.memberOf.value = $groupDN
				}
				else
				{
					# create a new <value> node
					$NewGroupValue = $xml.CreateElement("value")
					$NewGroupValue.set_InnerXML("$groupDN")
					[void]$NewGroup.memberOf.AppendChild($NewGroupValue)
				}
			}
		}

		# call the function to add the node
		AddChildXML $XMLParentNode $NewGroup
	}
}
Write-Host "*******************************"

##########################################################################################################
#Search the customer OU for all contacts, save results to xml file

$filter = "(&(objectcategory=Person)(objectClass=Contact))"
$searchRoot = "LDAP://OU=$CustomerID,$BaseSearchPath"
Write-Host "Searching for Contacts in Customer OU: " $searchRoot -foregroundcolor cyan

#call the SearchLDAP function
$searchResults = SearchLDAP $searchRoot $filter

if ($searchResults.length -gt 0)
{
	foreach ($objResult in $searchResults)
	{
		# clear out the dynamic array for contact's proxy addresses
		$arrContactProxyAddrs = @()
	
		# clear out the dynamic array for the contact's 'member of' values (if any)
		$arrContactMemberOf = @()

		# list each property you want to retrieve.  attribute must be specified in all lower case!
		$objItem = $objResult.Properties
		[string]$ContactCN = $objItem.cn
		[string]$ContactDN = $objItem.distinguishedname
		[string]$ContactDisplayName = $objItem.displayname
		[string]$ContactName = $objItem.name
		[string]$ContactAlias = $objItem.mailnickname
		[string]$ContactLED = $objItem.legacyexchangedn
		[string]$ContactTargetAddr = $objItem.targetaddress
		$arrContactProxyAddrs = $objItem.proxyaddresses
		$arrContactMemberOf = $objItem.memberof

		Write-Host $ContactName
		$ContactsFound += 1

		### Save the contact account info into the XML dom

		# set the XML parent node
		$XMLParentNode = $xml.response.data.contacts

		# select the node to clone
		$XMLContactNode = @($XMLParentNode.contact)[0]

		# clone the node
		$NewContact = $XMLContactNode.Clone()

		# Set the values of the new node
		$NewContact.cn = $ContactCN
		$NewContact.distinguishedName = $ContactDN
		$NewContact.displayName = $ContactDisplayName
		$NewContact.targetAddress = $ContactTargetAddr
		$NewContact.legacyExchangeDN = $ContactLED
		$NewContact.Name = $ContactName
		if ($ContactAlias -gt "")
		{
			$MailContactsFound += 1
			$NewContact.mailEnabled = "TRUE"
			$NewContact.mailNickname = $ContactAlias
		}
		Else
		{
			$NewContact.mailEnabled = "FALSE"
		}

		if ($arrContactProxyAddrs -gt "")
		{
			foreach ($proxyAddr in $arrContactProxyAddrs)
			{
				if ($NewContact.proxyAddresses.value -eq "")
				{
					# there is an empty <value> node we can use
					$NewContact.proxyAddresses.value = $proxyAddr
				}
				else
				{
					# create a new <value> node
					$NewValue = $xml.CreateElement("value")
					$NewValue.set_InnerXML("$proxyAddr")
					[void]$NewContact.proxyAddresses.AppendChild($NewValue)
				}
			}
			
		}
		if ($arrContactMemberOf -gt "")
		{
			foreach ($groupDN in $arrContactMemberOf)
			{
				if ($NewContact.memberOf.value -eq "")
				{
					# there is an empty <value> node we can use
					$NewContact.memberOf.value = $groupDN
				}
				else
				{
					# create a new <value> node
					$NewContactValue = $xml.CreateElement("value")
					$NewContactValue.set_InnerXML("$groupDN")
					[void]$NewContact.memberOf.AppendChild($NewContactValue)
				}
			}
		}

		# call the function to add the node
		AddChildXML $XMLParentNode $NewContact
	}
}
Write-Host "*******************************"

##########################################################################################################

# remove empty template nodes from the xml results file

$XMLTemplateSMTPDomainNode = @($xml.response.data.SMTPDomains.SMTPDomain)[0]
[void]$xml.response.data.SMTPDomains.RemoveChild($XMLTemplateSMTPDomainNode)

$XMLTemplateUserNode = @($xml.response.data.users.user)[0]
[void]$xml.response.data.users.RemoveChild($XMLTemplateUserNode)

$XMLTemplateGroupNode = @($xml.response.data.groups.group)[0]
[void]$xml.response.data.groups.RemoveChild($XMLTemplateGroupNode)

$XMLTemplateContactNode = @($xml.response.data.contacts.contact)[0]
[void]$xml.response.data.contacts.RemoveChild($XMLTemplateContactNode)

############################################
#         Logging and Reporting            #
############################################

Write-Host "#############################################################" -foregroundcolor cyan
Write-Host "Found $SMTPDomainsFound SMTP domains" -foregroundcolor cyan
Write-Host "Found $UsersFound Users, of which $MailboxesFound are mailbox-enabled (not including System mailboxes)" -foregroundcolor cyan
Write-Host "Found $GroupsFound Groups, of which $MailGroupsFound are mail-enabled (not including System groups)" -foregroundcolor cyan
Write-Host "Found $ContactsFound Contacts, of which $MailContactsFound are mail-enabled" -foregroundcolor cyan
Write-Host "#############################################################" -foregroundcolor cyan

# write out the XML query results file
$xml.Save($OutputFile)
Write-Host ""
Write-Host "=============================================================" -foregroundcolor cyan
Write-Host "Saved results to XML output file $OutputFile" -foregroundcolor cyan
Write-Host "=============================================================" -foregroundcolor cyan
