#--------------------------------------------------------------------------------
# FILE:       AssignABPToCustomerMailboxes.ps1
#
# PROJECT:    Exchange 2010 /Hosting mode to Exchange 2010 SP2 Migration
#
# PURPOSE:	Sample script to apply an Address Book Policy to mailboxes
#		in the tenant's OU.  Do not run this script until all tenant
#		mailboxes have been moved to Exchange 2010 SP2 mailbox servers
#
# THIS CODE AND INFORMATION IS PROVIDED TO YOU FOR YOUR REFERENTIAL PURPOSES
# ONLY, AND IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EITHER
# EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE, AND MAY NOT
# BE REDISTRIBUTED IN ANY MANNER.
#
# Copyright (C) 2012 Microsoft Corporation.  All rights reserved
#--------------------------------------------------------------------------------

### Modify these variables with values appropriate to your environment ###

$PreferredDC = "DC01.sp2enterprise.net"
$DomainID = "sp2enterprise"
$DomainSuffix = "net"
$HostingID = "Hosting"
$ResellerID = "Reseller001"

$BaseADPath = "OU=$ResellerID,OU=$HostingID,DC=$DomainID,DC=$DomainSuffix"

################### You should not need to edit any variables below this line ###################

if ($args.length -ne 1)
{
	Write-Host "You must provide the name of the customer OU as an argument." -foregroundcolor yellow
	Write-Host "if there are spaces or special characters in the name, surround it with quotes." -foregroundcolor yellow
	Write-Host "Exiting script." -foregroundcolor yellow
	exit
}
$TenantID = $args[0]

$MailObjectsFound = 0
$MailObjectsUpdated = 0
$MailObjectsFailed = 0

# open logfile
$Timestamp = get-date -f yyyy-mm-dd-hhmmss
$Logfile = "AssignABPToMailboxesLog-" + "$TenantID-" + $Timestamp + ".txt"
get-date | out-file $Logfile -encoding ASCII
"Beginning bulk Assignment of ABP to customer Mailboxes." | out-file $Logfile -append -encoding ASCII
"-------------------------------------------" | out-file $Logfile -append -encoding ASCII

### Retrieve a list of all mailboxes in the customer OU
Write-Host "Retrieving a list of all mailboxes in the customer OU..." -foregroundcolor cyan
$MailboxArray = Get-Mailbox -OrganizationalUnit "OU=$TenantID,$BaseADPath" -DomainController $PreferredDC 

foreach ($MailObject in $MailboxArray)
{ 
	$MailObjectsFound += 1
	#Clear Error Count
	$error.clear()
	$oldErrCount = $error.Count

	### Assign the Address Book Policy to each mailbox
	#Clear Error Count
	$error.clear()
	$oldErrCount = $error.Count
	Write-Host "Applying Address Book Policy $TenantID ABP to $MailObject"
	"Attempting to apply Address Book Policy $TenantID ABP to $MailObject" | out-file $Logfile -append -encoding ASCII 
	Set-Mailbox -Identity $MailObject -AddressBookPolicy "$TenantID ABP" -DomainController $PreferredDC

	if ($error.Count -ne $oldErrCount)
	{
		[string]$ErrorInfo = $Error[$error.count-1].exception
		$MailObjectsFailed += 1
		$ErrorInfo  | out-file $Logfile -append -encoding ASCII 
		Write-Host "Unable to apply Address Book Policy to mailbox $MailObject" -foregroundcolor yellow
		"Unable to apply Address Book Policy to mailbox $MailObject" | out-file $Logfile -append -encoding ASCII 
		Write-Host "-----------------------------------------------------------"
		"-----------------------------------------------------------" | out-file $Logfile -append -encoding ASCII 
	}
	else
	{
		#successfully applied ABP to mailbox
		$MailObjectsUpdated += 1
		Get-Mailbox $MailObject | Select Name,AddressBookPolicy | ft -autosize
		"Successfully applied ABP to $MailObject" | out-file $Logfile -append -encoding ASCII 
		Write-Host "-----------------------------------------------------------"
		"-----------------------------------------------------------" | out-file $Logfile -append -encoding ASCII 
	}
	## Update the User
}
Write-Host "* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * "
Write-Host "Mailboxes found in tenant OU: $MailObjectsFound"
Write-Host "Mailboxes successfully had ABP applied: $MailObjectsUpdated"
Write-Host "Mailboxes failed to have ABP applied: $MailObjectsFailed"

# write results to logfile
"* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * " | out-file $Logfile -append -encoding ASCII 
"Mailboxes found in tenant OU: $MailObjectsFound" | out-file $Logfile -append -encoding ASCII 
"Mailboxes successfully had ABP applied: $MailObjectsUpdated" | out-file $Logfile -append -encoding ASCII 
"Mailboxes failed to have ABP applied: $MailObjectsFailed" | out-file $Logfile -append -encoding ASCII 
Write-Host "Review logfile $Logfile for details on any errors." -foregroundcolor cyan
