#--------------------------------------------------------------------------------
# FILE:       ClearChangePasswordFlag.ps1
#
# PROJECT:    Exchange 2010 /Hosting mode to Exchange 2010 SP2 Migration
#
# PURPOSE:    Sample script that parses the customer query output XML file
#             for user information, and clears the 'User must reset password on 
#             next login' flag that was set during the ADMT user migration. 
#
#
# THIS CODE AND INFORMATION IS PROVIDED TO YOU FOR YOUR REFERENTIAL PURPOSES 
# ONLY, AND IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EITHER 
# EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE, AND MAY NOT 
# BE REDISTRIBUTED IN ANY MANNER.
#
# Copyright (C) 2012 Microsoft Corporation. All rights reserved.
#--------------------------------------------------------------------------------

### Modify these variables with values appropriate to your environment ###

$PreferredDC = "DC01.sp2enterprise.net"
$DomainID = "sp2enterprise"
$DomainSuffix = "net"
$HostingID = "Hosting"
$ResellerID = "Reseller001"

$UserAccountControlValue = 66048 

$BaseADPath = "OU=$ResellerID,OU=$HostingID,DC=$DomainID,DC=$DomainSuffix"

################### You should not need to edit any variables below this line ###################

if ($args.length -ne 2)
{
	Write-Host "You must provide the name of the customer XML output file and the customer OU name as an argument." -foregroundcolor yellow
	Write-Host "if there are spaces or special characters either, surround each with quotes." -foregroundcolor yellow
	Write-Host "Exiting script." -foregroundcolor yellow
	exit
}

$OrgXMLFile = $args[0]
$CustomerID = $args[1]

$UsersFound = 0
$UsersFailed = 0
$UsersSkipped = 0

$Timestamp = get-date -f yyyy-mm-dd-hhmmss
$ErrorLogfile = "RestoreUserAttributesLog-" + $OrgID + "-" + $Timestamp + ".txt"

## Open error logfile
get-date | out-file $ErrorLogFile
"Beginning Bulk Restore of user attributes..." | out-file $ErrorLogfile -append
"-------------------------------------------------" | out-file $ErrorLogfile -append

#################################
#Function to perform LDAP query 
#################################

function SearchLDAP ($searchRoot,$filter)
{
	$searcher = New-Object System.DirectoryServices.DirectorySearcher
	$searcher.SearchRoot = $searchRoot
	$searcher.PageSize = 1000
	$searcher.Filter = $filter
	$searcher.SearchScope = "Onelevel" # Set to Base, Onelevel or Subtree
	$searchResults = $searcher.FindAll() # Set to FindOne or FindAll

	# dispose of the AD searcher object in order to clean house
	$searcher.Dispose()

        if ($searchResults -eq $null -or $searchResults.count -eq 0)
        {
            return $null
        }
	else
	{
		return $searchResults
	}
}
##################################

# pull in the data from input XML file
$OrgData = [xml](Get-Content $OrgXMLFile)

# parse the XML file and find all the User nodes
ForEach($User in $OrgData.GetElementsByTagName("user"))
{
	$UsersFound += 1
	$Name = $User.name
	Write-Host "Processing User $Name" -foregroundcolor green
	"Processing User $Name" | out-file $ErrorLogfile -append
	$UPN = $User.userPrincipalName
	$CN = $User.cn

	# call the SearchLDAP function so we can bind to the user in AD

	$filter = "(&(objectClass=User)(name=$Name))"
	$searchRoot = "LDAP://OU=$CustomerID,$BaseADPath"
	$searchResults = SearchLDAP $searchRoot $filter

	# clear the "User must reset password at next logon flag" by changing pwdLastSet to -1
	### Optionally, restore each user's UPN, and set their UserAccountControl value
	foreach ($objResult in $searchResults)
	{
		$objItem = $objResult.Properties
		Write-Host "    Clearing the password reset flag for $Name"
		$UserAccount = $objResult.GetDirectoryEntry()

		$UserAccount.Put("pwdLastSet",-1)
		#$UserAccount.Put("userPrincipalName",$UPN)
		#$UserAccount.Put("userAccountControl",$UserAccountControlValue)
		$UserAccount.SetInfo()
		$UsersAssociated += 1
	}
}
# write output to screen
Write-Host "============================================================="
Write-Host "Users found in query results file: $UsersFound"
Write-Host "Users who had password reset flag cleared: $UsersAssociated"
Write-Host ""
Write-Host "Any errors can be reviewed in logfile $ErrorLogfile"

# write output to logfile
"=============================================================" | out-file $ErrorLogfile -append
"Users found in query results file: $UsersFound" | out-file $ErrorLogfile -append
"Users with UPN and other values restored: $UsersAssociated" | out-file $ErrorLogfile -append
