#--------------------------------------------------------------------------------
# FILE:       CreateCustomerAddressLists.ps1
#
# PROJECT:    Exchange 2010 /Hosting mode to Exchange 2010 SP2 Migration
#
# PURPOSE:    Sample script that creates Address Lists and Address Book Policy (ABP)
#             for a customer organization that is going to be migrated.
#
#
# THIS CODE AND INFORMATION IS PROVIDED TO YOU FOR YOUR REFERENTIAL PURPOSES 
# ONLY, AND IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EITHER 
# EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE, AND MAY NOT 
# BE REDISTRIBUTED IN ANY MANNER.
#
# Copyright (C) 2012 Microsoft Corporation. All rights reserved.
#--------------------------------------------------------------------------------

### Modify these variables with values appropriate to your environment ###

$PreferredDC = "DC01.sp2enterprise.net"
$DomainID = "sp2enterprise"
$DomainSuffix = "net"
$HostingID = "Hosting"
$ResellerID = "Reseller001"

$OABServer = "Ex2010SP2MBX"

$BaseADPath = "OU=$ResellerID,OU=$HostingID,DC=$DomainID,DC=$DomainSuffix"

################### You should not need to edit any variables below this line ###################

if ($args.length -ne 1)
{
	Write-Host "You must provide the name of the customer organization as an argument." -foregroundcolor yellow
	Write-Host "if there are spaces or special characters in the name, surround it with quotes." -foregroundcolor yellow
	Write-Host "Exiting script." -foregroundcolor yellow
	exit
}
$CustomerXMLFile = $args[0]

## Open error logfile
$Timestamp = get-date -f yyyy-mm-dd-hhmmss
$Logfile = "CreateCustomerAddressListsLog-" + "$TenantID-" + $Timestamp + ".txt"
get-date | out-file $LogFile
"Beginning creation of Address Lists and ABP for customer $TenantID..." | out-file $Logfile -append
"-------------------------------------------------" | out-file $Logfile -append

##### import the customer xml output file
$error.clear()
$oldErrorCount = $error.Count
$xml = [xml](get-content "C:\Scripts\$CustomerXMLFile") 

if ($error.Count -ne $oldErrorCount)
{
	[string]$problem = $error[0].Exception
	if ($problem.contains("because it does not exist"))
	{
		write-host "Unable to find C:\Scripts\$CustomerXMLFile" -foregroundcolor yellow
		write-host "this file must be present in the C:\Scripts directory in order for the script to run." -foregroundcolor yellow
		write-host "Exiting script." -foregroundcolor yellow
		exit
	}
	Else
	{
		write-host $error[0].Exception -foregroundcolor red
		exit
	}
}

##### parse the XML file for customer name
$TenantID = $xml.response.data.customerOU
Write-Host "Customer Name in XML file is: $TenantID" -foregroundcolor cyan

##### Create Tenant Global Address List

$Filter1 = "CustomAttribute1 -eq '"+$TenantID+"'"
$Filter2 = "Alias -ne '`$null'"
Write-Host "Creating $TenantID Global Address List" -foregroundcolor cyan
"Creating $TenantID Global Address List" | out-file $Logfile -append -encoding ASCII
#Clear Error Count
$error.clear()
$oldErrCount = $error.Count
$GAL = New-GlobalAddressList -Name "$TenantID GAL" -RecipientFilter "$Filter1 -and $Filter2" -DomainController $PreferredDC
if ($error.Count -ne $oldErrCount)
{
	out-file $Logfile -InputObject $Error[$error.count-1].exception -append -width 119 -encoding ASCII
	$GALCreationErrors += 1
}
Else
{
	$GAL.Name
	#$GAL.RecipientFilter
}

##### Create All Rooms AL
Write-Host "Creating $TenantID All Rooms AL" -foregroundcolor cyan
"Creating $TenantID All Rooms AL" | out-file $Logfile -append -encoding ASCII
$RoomsAL = New-AddressList -Name "$TenantID All Rooms AL" -Container "\" -DisplayName "$TenantID All Rooms AL" -recipientfilter "$Filter2 -and (((RecipientDisplayType -eq 'ConferenceRoomMailbox') -or (RecipientDisplayType -eq 'SyncedConferenceRoomMailbox')) -and ($Filter1))" -DomainController $PreferredDC
$RoomsAL.Name
#$RoomsAL.RecipientFilter

##### Create All Users AL
Write-Host "Creating $TenantID All Users AL" -foregroundcolor cyan
"Creating $TenantID All Users AL" | out-file $Logfile -append -encoding ASCII
$UsersAL = New-AddressList -Name "$TenantID All Users AL" -Container "\" -DisplayName "$TenantID All Users AL" -RecipientFilter "(($Filter2) -and (ObjectClass -eq 'user') -and ($Filter1))" -DomainController $PreferredDC
$UsersAL.Name
#$UsersAL.RecipientFilter

##### Create All Contacts AL
Write-Host "Creating $TenantID All Contacts AL" -foregroundcolor cyan
"Creating $TenantID All Contacts AL"| out-file $Logfile -append -encoding ASCII
$ContactsAL = New-AddressList -Name "$TenantID All Contacts AL" -Container "\" -DisplayName "$TenantID All Contacts AL" -RecipientFilter "(($Filter2) -and (ObjectClass -eq 'contact') -and ($Filter1))" -DomainController $PreferredDC
$ContactsAL.Name
#$ContactsAL.RecipientFilter

##### Create All Groups AL
Write-Host "Creating $TenantID All Groups AL" -foregroundcolor cyan
"Creating $TenantID All Groups AL" | out-file $Logfile -append -encoding ASCII
$GroupsAL = New-AddressList -Name "$TenantID All Groups AL" -Container "\" -DisplayName "$TenantID All Groups AL" -RecipientFilter "(($Filter2) -and (ObjectCategory -like 'group') -and ($Filter1))" -DomainController $PreferredDC
$GroupsAL.Name
#$GroupsAL.RecipientFilter

##### Create Offline Addres Book (OAB)
Write-Host "Searching for available Exchange 2010 CAS OAB virtual directories..." -foregroundcolor cyan

[array]$OABVdirs = Get-OABVirtualDirectory -DomainController $PreferredDC

foreach ($i in $OABVdirs)
{
	$Server = $i.Server 
	$VDir = $i.Name
	$Version = $i | Select-Object -ExpandProperty ExchangeVersion
	$MajorVersion = $Version.ExchangeBuild.Major
				
	#Write-Host "Server $Server is version $MajorVersion. VDir is: $VDIR"
		
	if ($MajorVersion -ge 14)
	{
		$FullOABVdirPath = "$Server\$VDir"
		Write-Host $FullOABVdirPath 
		$2010OABVdirs += $FullOABVdirPath 
	}
}
Write-Host "---------------------------------------"
Write-Host "Will configure OAB to be distributed by the following OAB VDirs:"
$2010OABVdirs
Write-Host "Will home OAB on mailbox server $OABServer"
Write-Host "Creating $TenantID OAB" -foregroundcolor cyan
"Creating $TenantID OAB homed on $OABServer" | out-file $Logfile -append -encoding ASCII
$OAB = New-OfflineAddressBook -Name "$TenantID OAB" -Server $OABServer -AddressLists "\$TenantID GAL" -VirtualDirectories $2010OABVdirs -DomainController $PreferredDC
$OAB.Name
Write-Host "Setting OAB Update Schedule to Never" -foregroundcolor cyan
"Setting OAB Update Schedule to Never" | out-file $Logfile -append -encoding ASCII
Set-OfflineAddressBook -Identity "$TenantID OAB" -Schedule Never -DomainController $PreferredDC

##### Create Address Book Policy
Write-Host "Creating $TenantID Address Book Policy" -foregroundcolor cyan
"Creating $TenantID Address Book Policy" | out-file $Logfile -append -encoding ASCII
$ABP = New-AddressBookPolicy -Name "$TenantID ABP" -GlobalAddressList "\$TenantID GAL" -OfflineAddressBook "\$TenantID OAB" -RoomList "\$TenantID All Rooms AL" -AddressLists "\$TenantID All Users AL","\$TenantID All Groups AL","\$TenantID All Contacts AL" -DomainController $PreferredDC
$ABP.Name

###################################################
Write-Host "-------------------------------------------"
"-------------------------------------------" | out-file $Logfile -append -encoding ASCII

Write-Host "Addres Lists  and ABP created for customer $TenantID." -foregroundcolor green
"Addres Lists  and ABP created for customer $TenantID ." | out-file $Logfile -append -encoding ASCII
Write-Host "######################################################################"
"######################################################################" | out-file $Logfile -append -encoding ASCII


get-date | out-file $Logfile -append -encoding ASCII
"Completed bulk creation of tenants." | out-file $Logfile -append -encoding ASCII
"-------------------------------------------" | out-file $Logfile -append -encoding ASCII
"Total GAL Creation Errors: $GALCreationErrors" | out-file $Logfile -append -encoding ASCII

