#--------------------------------------------------------------------------------
# FILE:       CreateCustomerOU.ps1
#
# PROJECT:    Exchange 2010 /Hosting mode to Exchange 2010 SP2 Migration
#
# PURPOSE:    Sample script that parses the customer query output XML file
#             for OU and UPN information, and creates a new customer OU in 
#             the Exchange 2010 SP2 forest. 
#
#
# THIS CODE AND INFORMATION IS PROVIDED TO YOU FOR YOUR REFERENTIAL PURPOSES 
# ONLY, AND IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EITHER 
# EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE, AND MAY NOT 
# BE REDISTRIBUTED IN ANY MANNER.
#
# Copyright (C) 2012 Microsoft Corporation. All rights reserved.
#--------------------------------------------------------------------------------

### Modify these variables with values appropriate to your environment ###

$PreferredDC = "DC01.sp2enterprise.net"
$DomainID = "sp2enterprise"
$DomainSuffix = "net"
$HostingID = "Hosting"
$ResellerID = "Reseller001"

$BaseADPath = "OU=$ResellerID,OU=$HostingID,DC=$DomainID,DC=$DomainSuffix"

################### You should not need to edit any variables below this line ###################

if ($args.length -ne 1)
{
	Write-Host "You must provide the name of the customer XML output file as an argument." -foregroundcolor yellow
	Write-Host "if there are spaces or special characters in the filename, surround it with quotes." -foregroundcolor yellow
	Write-Host "Exiting script." -foregroundcolor yellow
	exit
}
$CustomerXMLFile = $args[0]

Import-Module ActiveDirectory -ea SilentlyContinue

##### import the customer xml output file
$error.clear()
$oldErrorCount = $error.Count
$xml = [xml](get-content "C:\Scripts\$CustomerXMLFile") 

if ($error.Count -ne $oldErrorCount)
{
	[string]$problem = $error[0].Exception
	if ($problem.contains("because it does not exist"))
	{
		write-host "Unable to find C:\Scripts\$CustomerXMLFile" -foregroundcolor yellow
		write-host "this file must be present in the C:\Scripts directory in order for the script to run." -foregroundcolor yellow
		write-host "Exiting script." -foregroundcolor yellow
		exit
	}
	Else
	{
		write-host $error[0].Exception -foregroundcolor red
		exit
	}
}

##### parse the XML file for required attributes
$CustomerOU = $xml.response.data.customerOU
Write-Host "CustomerOU Name in XML file is: $CustomerOU" -foregroundcolor cyan

# create a dynamic array to hold customer SMTP domains
$arrCustomerSMTPDomains = @()

foreach ($node in $xml.GetElementsByTagName("SMTPDomain"))
{
	$CustomerSMTPDomain = $node.domainName
	$arrCustomerSMTPDomains += $CustomerSMTPDomain
	Write-Host "found SMTP domain $CustomerSMTPDomain.  Will add this as a UPNSuffix on the customer OU." -foregroundcolor cyan
}

##### CREATE THE CUSTOMER OU
$error.clear()
$oldErrorCount = $error.Count

Write-Host "Creating OU for $CustomerOU" -foregroundcolor cyan
New-ADOrganizationalUnit -Name $CustomerOU -Path $BaseADPath -OtherAttributes @{uPNSuffixes=$arrCustomerSMTPDomains} -ProtectedFromAccidentalDeletion $false -Server $PreferredDC

if ($error.Count -ne $oldErrorCount)
{
	[string]$problem = $error[0].Exception
	if ($problem.contains("name that is already in use"))
	{
		write-host "OU $CustomerOU already exists!" -foregroundcolor yellow
		write-host "Exiting script." -foregroundcolor yellow
		exit
	}
	Else
	{
		write-host $error[0].Exception -foregroundcolor red
		exit
	}
}
else
{
	Get-ADOrganizationalUnit -Identity "OU=$CustomerOU,$BaseADPath" -Server $PreferredDC
}