#--------------------------------------------------------------------------------
# FILE:       ImportOrgContacts.ps1
#
# PROJECT:    Exchange 2010 /Hosting mode to Exchange 2010 SP2 Migration
#
# PURPOSE:    Sample script that parses the customer query output XML file
#             for mail-enabled Contact information, creates those contacts in the  
#             target domain, and restores group membership (if any)
#
#
# THIS CODE AND INFORMATION IS PROVIDED TO YOU FOR YOUR REFERENTIAL PURPOSES 
# ONLY, AND IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EITHER 
# EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE, AND MAY NOT 
# BE REDISTRIBUTED IN ANY MANNER.
#
# Copyright (C) 2012 Microsoft Corporation. All rights reserved.
#--------------------------------------------------------------------------------

### Modify these variables with values appropriate to your environment ###

$PreferredDC = "DC01.sp2enterprise.net"
$DomainID = "sp2enterprise"
$DomainSuffix = "net"
$HostingID = "Hosting"
$ResellerID = "Reseller001"

$BaseADPath = "OU=$ResellerID,OU=$HostingID,DC=$DomainID,DC=$DomainSuffix"

################### You should not need to edit any variables below this line ###################

if ($args.length -ne 2)
{
	Write-Host "You must provide the name of the customer XML output file and the customer OU name as an argument." -foregroundcolor yellow
	Write-Host "if there are spaces or special characters either, surround each with quotes." -foregroundcolor yellow
	Write-Host "Exiting script." -foregroundcolor yellow
	exit
}

$OrgXMLFile = $args[0]
$CustomerID = $args[1]

$ContactsFound = 0
$ContactsCreated = 0
$ContactsFailed = 0
$ContactsSkipped = 0

## Define a dynamic array to hold groups
$arrMemberOf = @()

## Define a dynamic array to hold email addresses
$arrProxyAddresses = @()

$Timestamp = get-date -f yyyy-mm-dd-hhmmss
$ErrorLogfile = "ImportOrgContactsLog-" + "$OrgID-" + $Timestamp + ".txt"

## Open error logfile
get-date | out-file $ErrorLogFile
"Beginning Bulk Import of Organization Contacts..." | out-file $ErrorLogfile -append
"-------------------------------------------------" | out-file $ErrorLogfile -append

Write-Host "Importing Contacts" -foregroundcolor green
"Importing Contacts" | out-file $ErrorLogfile -append

# pull in the data from input XML file
$OrgData = [xml](Get-Content $OrgXMLFile)

# parse the XML file and find all the SMTPDomain nodes

$arrSMTPDomains = @()
ForEach($domain in $OrgData.GetElementsByTagName("SMTPDomain"))
{
	$Name = $domain.domainName
	$arrSMTPDomains += $Name
}

$InternalDomain = $arrSMTPDomains[0]
Write-Host "     Selecting SMTP domain $InternalDomain"

# parse the XML file and find all the Contact nodes
ForEach($Contact in $OrgData.GetElementsByTagName("contact"))
{
	$ContactsFound += 1
	# Clear out the Group Membership array, as we have moved on to a new user
	$arrMemberOf = @()
	# Clear out the email addresses array, as we have moved on to a new user
	$arrProxyAddresses = @()

	$Name = $Contact.name
	Write-Host "Beginning Import of contact $Name" -foregroundcolor green
	"Beginning Import of contact $Name" | out-file $ErrorLogfile -append
	$DisplayName = $Contact.displayName
	$Alias = $Contact.mailnickname
	$TargetAddress = $Contact.targetAddress
	$LegacyExchangeDN = "X500:" + $Contact.legacyexchangedn

	ForEach($group in $contact.memberOf.value)
	{
		if ($group -ne $null)
		{
			$arrMemberOf += $group
		}
	}

	if ($Alias.length -gt 0) # we only care about mail-enabled Contacts
	{
		## Clear the Error Count
		$error.clear()
		$oldErrorCount = $error.Count

		# Construct the unique internal proxy address
		$InternalProxy = $Alias + "_contact@" + $InternalDomain
		Write-Host "     Unique internal proxy will be: $InternalProxy" -foregroundcolor magenta
		$arrProxyAddresses += $InternalProxy
		
		#Create the mail-enabled Contact
		Write-Host "     Mail-enabling contact $Name" -foregroundcolor cyan
		New-MailContact -Name $Name -Alias $Alias -DisplayName $DisplayName -ExternalEmailAddress $TargetAddress -PrimarySmtpAddress $InternalProxy -OrganizationalUnit "OU=$CustomerID,$BaseADPath" -DomainController $PreferredDC -erroraction silentlycontinue
		if ($error.Count -ne $oldErrorCount)
		{
			[string]$problem = $error[0].Exception
			if ($problem.contains("already exists"))
			{
				$ContactsSkipped += 1
				# write output to screen and logfile
				write-host "    contact $Name already exists" -foregroundcolor yellow
				"    contact $Name already exists" | out-file $ErrorLogfile -append
			}
			Else
			{
				$ContactsFailed += 1
				# write output to screen and logfile
				write-host $error[0].Exception -foregroundcolor red
				$error[0].Exception | out-file -append
			}
		}
		Else
		{
			$ContactsCreated += 1
		}

		# Step through any secondary proxy addresses the contact may have
		ForEach($proxy in $Contact.proxyAddresses.value)
		{
			Write-Host "     Found proxy address $proxy" -foregroundcolor magenta
			#break up the proxy address so we can determine what type it is
			$arrProxyInfo = $proxy.split(":")
			$ProxyHeader = $arrProxyInfo[0]
			$ProxyAddr = $arrProxyInfo[1]

			# use 'Switch' statement to handle various types of proxy address
			Switch -casesensitive ($ProxyHeader)
			{
				"SMTP"
				{
					Write-Host "     Found SMTP"
					# We don't add target adress as a proxy, because it wouldn't be unique
				}
				
			
				"X500"
				{
					Write-Host "     Found X500"
					# we  convert this to a secondary x500 address, because later on we'll set legacyExchangeDN as primary X500.
					$proxy = $proxy -replace "X500","x500"
					$arrProxyAddresses += $proxy
				}
				
				"x500"
				{
					Write-Host "     found x500"
					$arrProxyAddresses += $proxy
				}
			}
		}

		#Add the LegacyExchangeDN as an X.500 proxy address
		$arrProxyAddresses += $LegacyExchangeDN
		
		Write-Host "The secondary proxy array is: $arrProxyAddresses" -foregroundcolor magenta

		## Clear the Error Count
		$error.clear()
		$oldErrorCount = $error.Count

		# Stamp secondary proxy addresses (including X500) on the contact
		Write-Host "     Stamping secondary proxy addresses on contact" -foregroundcolor cyan
		Set-MailContact -Identity $Name -EmailAddresses $arrProxyAddresses -DomainController $PreferredDC 
		if ($error.Count -ne $oldErrorCount)
		{
			[string]$problem = $error[0].Exception
			if ($problem.contains("already exists"))
			{
				# write output to screen and logfile
				write-host "    skipping $Name - object already exists" -foregroundcolor yellow
				"    skipping $Name - object already exists" | out-file $ErrorLogfile -append
			}
			Else
			{
				# write output to screen and logfile
				write-host $error[0].Exception -foregroundcolor red
				$error[0].Exception | out-file -append
			}
		}
		Else
		{
			$ContactsUpdated += 1
		}

		## now lets add the contact back into its groups (if any)
		if ($arrMemberOf -ne $null)
		{
			foreach ($group in $arrMemberOf)
			{
				# group is currently identified by DN in source (/Hosting forest)
				# need to convert that to Name to simplify adding user to group in target forest
				$arrGroupInfo = $group.Split("=")
				$GroupName = $arrGroupInfo[1]
				$GroupName = $GroupName.Replace(",OU", "")

				write-host "    Adding $Name to $GroupName"
				"    Adding $Name to $GroupName" | out-file $ErrorLogfile -append

				## Clear the Error Count
				$error.clear()
				$oldErrorCount = $error.Count
				Add-DistributionGroupMember $GroupName -Member $Alias -DomainController $PreferredDC -erroraction silentlycontinue

				if ($error.Count -ne $oldErrorCount)
				{
					[string]$problem = $error[0].Exception
					if ($problem.contains("is already a member of the group"))
					{
						write-host "    $Name is already a member of $HostedGroup" -foregroundcolor yellow
						"     $Name is already a member of $HostedGroup" | out-file $ErrorLogfile -append
					}
					Else
					{
						write-host $error[0].Exception -foregroundcolor red
					}
				}
			}
		}
	}
}

# write output to screen
Write-Host "============================================================="
Write-Host "Contacts Created: $ContactsCreated"
Write-Host "Contact Skipped because they already exist: $ContactsSkipped"
Write-Host "Contacts updated with secondary proxy addresses: $ContactsUpdated"
Write-Host "Contacts failed to create: $ContactsFailed"
Write-Host ""
Write-Host "Any errors can be reviewed in logfile $ErrorLogfile"

# write output to logfile
"=============================================================" | out-file $ErrorLogfile -append
"Contacts Created: $ContactsCreated" | out-file $ErrorLogfile -append
"Contact Skipped because they already exist: $ContactsSkipped" | out-file $ErrorLogfile -append
"Contacts updated with secondary proxy addresses: $ContactsUpdated" | out-file $ErrorLogfile -append
"Contacts failed to create: $ContactsFailed" | out-file $ErrorLogfile -append
