#--------------------------------------------------------------------------------
# FILE:       ImportOrgSMTPDomains.ps1
#
# PROJECT:    Exchange 2010 /Hosting mode to Exchange 2010 SP2 Migration
#
# PURPOSE:    Sample script that parses the customer query output XML file
#             for customer SMTP domains information, and adds those as 
#             accepted domains in the target Exchange 2010 SP2 forest.
#
#
# THIS CODE AND INFORMATION IS PROVIDED TO YOU FOR YOUR REFERENTIAL PURPOSES 
# ONLY, AND IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EITHER 
# EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE, AND MAY NOT 
# BE REDISTRIBUTED IN ANY MANNER.
#
# Copyright (C) 2012 Microsoft Corporation. All rights reserved.
#--------------------------------------------------------------------------------

### Modify these variables with values appropriate to your environment ###

$PreferredDC = "dc01.sp2enterprise.net"

################### You should not need to edit any variables below this line ###################

if ($args.length -ne 1)
{
	Write-Host "You must provide the name of the customer query XML file as an argument." -foregroundcolor yellow
	Write-Host "if there are spaces or special characters in the filename, surround it with quotes." -foregroundcolor yellow
	Write-Host "Exiting script." -foregroundcolor yellow
	exit
}

$OrgXMLFile = $args[0]

$SMTPDomainsFound = 0
$SMTPDomainsCreated = 0
$SMTPDomainsFailed = 0
$SMTPDomainsSkipped = 0

$Timestamp = get-date -f yyyy-mm-dd-hhmmss
$ErrorLogfile = "Import-Org-SMTPDomains-Log-" + $Timestamp + ".txt"

## Open error logfile
get-date | out-file $ErrorLogFile
"Beginning Bulk Import of Organization SMTPDomains..." | out-file $ErrorLogfile -append
"-------------------------------------------------" | out-file $ErrorLogfile -append

# pull in the data from input XML file
$OrgData = [xml](Get-Content $OrgXMLFile)

# parse the XML file and find all the SMTPDomain nodes
ForEach($domain in $OrgData.GetElementsByTagName("SMTPDomain"))
{
	$SMTPDomainsFound += 1
	$Name = $domain.domainName
	$FriendlyName = $domain.domainFriendlyName
	$Type = $domain.domainType

	# logging and reporting
	Write-Host "Found SMTP domain $Name" -foregroundcolor cyan
	Write-Host "     Friendly Name: $FriendlyName"
	Write-Host "     domain Type: $Type"
	"Found SMTP domain $Name" | out-file $ErrorLogfile -append
	"     Friendly Name: $FriendlyName" | out-file $ErrorLogfile -append
	"     domain Type: $Type" | out-file $ErrorLogfile -append

	## Clear the Error Count
	$error.clear()
	$oldErrorCount = $error.Count

	#Create the SMTPDomain
	Write-Host "Adding $Name as an accepted domain..." -foregroundcolor cyan
	"     Adding $Name as an accepted domain..." | out-file $ErrorLogfile -append
	
	New-AcceptedDomain -Name $FriendlyName -DomainName $Name -DomainType $Type -DomainController $PreferredDC -erroraction silentlycontinue
	if ($error.Count -ne $oldErrorCount)
	{
		[string]$problem = $error[0].Exception
		if ($problem.contains("already exists"))
		{
			$SMTPDomainsSkipped += 1
			# write output to screen and logfile
			write-host "    skipping $Name - this accepted domain already exists" -foregroundcolor yellow
			"    skipping domain $Name - this accepted domain already exists" | out-file $ErrorLogfile -append
		}
		Else
		{
			$SMTPDomainsFailed += 1
			# write output to screen and logfile
			write-host $error[0].Exception -foregroundcolor red
			$error[0].Exception | out-file -append
		}
	}
	Else
	{
		$SMTPDomainsCreated += 1
	}
}

# write output to screen
Write-Host "============================================================="
Write-Host "SMTPDomains found in the input file: $SMTPDomainsFound"
Write-Host "SMTPDomains Created: $SMTPDomainsCreated"
Write-Host "SMTPDomain Skipped because they already exist: $SMTPDomainsSkipped"
Write-Host "SMTPDomains failed to create: $SMTPDomainsFailed"
Write-Host ""
Write-Host "Any errors can be reviewed in logfile $ErrorLogfile"

# write output to logfile
"=============================================================" | out-file $ErrorLogfile -append
"SMTPDomains found in the input file: $SMTPDomainsFound" | out-file $ErrorLogfile -append
"SMTPDomains Created: $SMTPDomainsCreated" | out-file $ErrorLogfile -append
"SMTPDomain Skipped because they already exist: $SMTPDomainsSkipped" | out-file $ErrorLogfile -append
"SMTPDomains failed to create: $SMTPDomainsFailed" | out-file $ErrorLogfile -append
