#--------------------------------------------------------------------------------
# FILE:       RestoreGroupAttributes.ps1
#
# PROJECT:    Exchange 2010 /Hosting mode to Exchange 2010 SP2 Migration
#
# PURPOSE:    Sample script that parses the customer query output XML file
#             for group information, and restores group attributes that were lost 
#             during the ADMT user migration.  This includes mail-enabling groups
#             that were mail-enabled in the source forest.
#
#
# THIS CODE AND INFORMATION IS PROVIDED TO YOU FOR YOUR REFERENTIAL PURPOSES 
# ONLY, AND IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EITHER 
# EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES 
# OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE, AND MAY NOT 
# BE REDISTRIBUTED IN ANY MANNER.
#
# Copyright (C) 2012 Microsoft Corporation. All rights reserved.
#--------------------------------------------------------------------------------

### Modify these variables with values appropriate to your environment ###

$PreferredDC = "dc01.sp2enterprise.net"
$DomainID = "SP2ENTERPRISE"

################### You should not need to edit any variables below this line ###################

if ($args.length -ne 1)
{
	Write-Host "You must provide the name of the customer XML output file as an argument." -foregroundcolor yellow
	Write-Host "if there are spaces or special characters in the filename, surround it with quotes." -foregroundcolor yellow
	Write-Host "Exiting script." -foregroundcolor yellow
	exit
}

$OrgXMLFile = $args[0]

$GroupsFound = 0
$MailEnabledGroupsFound = 0
$groupsCreated = 0
$GroupsAssociated = 0
$groupsFailed = 0
$groupsSkipped = 0
$groupsFailedToUpdateProxies = 0

## Define a dynamic array to hold email addresses
$arrProxyAddresses = @()

$Timestamp = get-date -f yyyy-mm-dd-hhmmss
$ErrorLogfile = "ImportOrgGroupsLog-" + $OrgID + "-" + $Timestamp + ".txt"

## Open error logfile
get-date | out-file $ErrorLogFile
"Beginning Bulk Import of Organization groups..." | out-file $ErrorLogfile -append
"-------------------------------------------------" | out-file $ErrorLogfile -append

##################################################################################################
# Function to check if the right-hand-side of a proxy address is an Accepted Domain 
##################################################################################################
Function CheckDomain ($ProxyDomain)
{
	Write-Host "     Checking to see if $ProxyDomain is an accepted domain..."
	
	# pull in the list of accepted domains 
	$arrAcceptedDomains = Get-AcceptedDomain -domaincontroller $PreferredDC

	# convert the domain to lowercase to make sure our comparison won't trip over case
	$LowerCaseProxyDomain = $ProxyDomain.ToLower()

	$arrDomainNames = @()
	foreach ($Entry in $arrAcceptedDomains)
	{
		# convert everything to lowercase to make sure our comparison won't trip over case
		$LowerCaseName = $Entry.name.ToLower()
		$arrDomainNames += $LowerCaseName
	}

	# search the array of accepted domains to see if the proxy domain is in it
	$Results = ($arrDomainNames -Contains $LowerCaseProxyDomain)
	#Write-Host "     proxy domain exists in accepted domains: $Results" -foregroundcolor yellow
	Return $Results
}
############################################################################################

# pull in the data from input XML file
$OrgData = [xml](Get-Content $OrgXMLFile)

# parse the XML file and find all the Group nodes
ForEach($Group in $OrgData.GetElementsByTagName("group"))
{
	$GroupsFound += 1
	# Clean out the proxy addresses array, as we have moved on to a new group
	$arrProxyAddresses = @()
	$Name = $Group.name
	Write-Host "Processing Group: $Name" -foregroundcolor green
	"Processing Group: " + $objItem.name | out-file $ErrorLogfile -append
	$DisplayName = $Group.displayName
	$Alias = $Group.mailNickname
	$LegacyExchangeDN = "X500:" + $Group.legacyExchangeDN
	$GroupSAM = $Group.sAMAccountName
	$FullGroupSAM = "$DomainID\$GroupSAM"

	Write-Host "     target group sAMAccountName is: $FullGroupSAM" -foregroundcolor cyan


	if ($Alias.length -gt 0) # we only care about mail-enabled Groups
	{
		$MailEnabledGroupsFound += 1
		Write-Host "     found Mail-enabled group $Name" -foregroundcolor green
		"     found Mail-enabled group $Name" | out-file $ErrorLogfile -append
		# Search through the array of proxy addresses to find primary SMTP proxy
		ForEach($proxy in $Group.proxyAddresses.value)
		{
			#Write-Host "     Found proxy address $proxy" -foregroundcolor magenta
			#break up the proxy address so we can determine what type it is
			$arrProxyInfo = $proxy.split(":")
			$ProxyHeader = $arrProxyInfo[0]
			$ProxyAddr = $arrProxyInfo[1]

			# use 'Switch' statement to handle various types of proxy address
			Switch -casesensitive ($ProxyHeader)
			{
				"SMTP"
				{
					#Write-Host "     Found SMTP"
					$PrimarySMTP = $ProxyAddr
					$arrProxyAddresses += $proxy
					
					$ProxyInfo = $ProxyAddr.split("@")
					$ProxyDomain = $ProxyInfo[1]					
					# call the function to make sure right-hand side of proxy address is in accepted domains for the tenant
					$CheckAcceptedDomains = CheckDomain $ProxyDomain

					If ($CheckAcceptedDomains -eq $True)
					{
						Write-Host "     Domain $ProxyDomain found in Accepted Domains"
						"     Domain $ProxyDomain found in Accepted Domains" | out-file $ErrorLogfile -append
					}
					Else
					{
						Write-Host "     domain for Primary SMTP address $ProxyAddr is not an accepted domain!" -foregroundcolor red
						"     domain for Primary SMTP address $ProxyAddr is not an accepted domain!" | out-file $ErrorLogfile -append
					}
				}
				
				"smtp"
				{
					#Write-Host "     Found smtp"
					$ProxyInfo = $ProxyAddr.split("@")
					$ProxyDomain = $ProxyInfo[1]					
					# call the function to make sure right-hand side of proxy address is in accepted domains for the tenant
					$CheckAcceptedDomains = CheckDomain $ProxyDomain

					If ($CheckAcceptedDomains -eq $True)
					{
						Write-Host "     Domain $ProxyDomain found in Accepted Domains"
						"      Domain $ProxyDomain found in Accepted Domains" | out-file $ErrorLogfile -append
						$arrProxyAddresses += $proxy
					}
				}
				
				"X500"
				{
					#Write-Host "     Found X500"
					# we  convert this to a secondary x500 address, because later on we'll set legacyExchangeDN as primary X500.
					$proxy = $proxy -replace "X500","x500"
					$arrProxyAddresses += $proxy
				}
				
				"x500"
				{
					#Write-Host "     found x500"
					$arrProxyAddresses += $proxy
				}
			}
		}
		
		#add the LegacyExchangeDN as an X.500 proxy address
		#Write-Host "     Adding legacyExchangeDN as X500 address"
		"     Adding legacyExchangeDN as X500 address" | out-file $ErrorLogfile -append
		$arrProxyAddresses += $LegacyExchangeDN	
		
		###### Mail-Enable the Group
		## Clear the Error Count
		$error.clear()
		$oldErrorCount = $error.Count

		Write-Host "     Mail-Enabling group $Name" -foregroundcolor cyan
		Enable-DistributionGroup $FullGroupSAM -Alias $Alias -DisplayName $DisplayName -PrimarySmtpAddress $PrimarySMTP -DomainController $PreferredDC -erroraction SilentlyContinue

		if ($error.Count -ne $oldErrorCount)
		{
			[string]$problem = $error[0].Exception
			if ($problem.contains("is already mail-enabled"))
			{
				$groupsSkipped += 1
				# write output to screen and logfile
				write-host "     Mail-enabled group $Name already exists" -foregroundcolor yellow
				"     Mail-enabled group $Name already exists" | out-file $ErrorLogfile -append
			}
			ElseIf ($problem.contains("couldnt be found on"))
			{
				$groupsFailed += 1
				# Write output to screen and logfile
				Write-Host "    Could not find group $Name.  Groups must be migrated with ADMT before you run this script to mail-enable them." -foregroundcolor red
				"     Could not find group $Name.  Groups must be migrated with ADMT before you run this script to mail-enable them." | out-file $ErrorLogfile -append
			}
			Else
			{
				$groupsFailed += 1
				# write output to screen and logfile
				write-host $error[0].Exception -foregroundcolor red
				$error[0].Exception | out-file -append
			}
		}
		Else
		{
			$groupsCreated += 1
		}

		###### Now we can use Set-DistributionList to add additional proxy addresses

		## Clear the Error Count
		$error.clear()
		$oldErrorCount = $error.Count

		# make sure the group actually got mail-enabled successfully before we try to modify it
		$CheckGroup = Get-DistributionGroup $FullGroupSAM -domaincontroller $PreferredDC -erroraction SilentlyContinue
		If ($CheckGroup -ne $Null)
		{
			Write-Host "     stamping secondary proxy addresses on group" -foregroundcolor cyan
			Set-DistributionGroup -Identity $FullGroupSAM -EmailAddresses $arrProxyAddresses -DomainController $PreferredDC 
		}
		Else
		{
			Write-Host "     Group not found. could not stamp secondary proxy addresses."
			"     Group not found. could not stamp secondary proxy addresses." | out-file $ErrorLogfile -append
		}
	}
}

# write output to screen
Write-Host "============================================================="
Write-Host "mail-enabled groups found in query results file: $MailEnabledGroupsFound"
Write-Host "groups mail-enabled: $groupsCreated"
Write-Host "groups not mail-enabled because they were already in that state: $groupsSkipped"
Write-Host "groups failed to mail-enable: $groupsFailed"
Write-Host ""
Write-Host "Any errors can be reviewed in logfile $ErrorLogfile"

# write output to logfile
"=============================================================" | out-file $ErrorLogfile -append
"mail-enabled groups found in query results file: $MailEnabledGroupsFound" | out-file $ErrorLogfile -append
"groups mail-enabled: $groupsCreated" | out-file $ErrorLogfile -append
"groups not mail-enabled because they were already in that state: $groupsSkipped" | out-file $ErrorLogfile -append
"groups failed to mail-enable: $groupsFailed" | out-file $ErrorLogfile -append
