#--------------------------------------------------------------------------------
# FILE:       StampCustomAttribute.ps1
#
# PROJECT:    Exchange 2010 /Hosting mode to Exchange 2010 SP2 Migration
#
# PURPOSE:	Sample script to modify the properties of a tenant's mail-
#		enabled Users, Contacts and Groups  in order to make them 
#		compatible with Exchange 2010 SP2 Address List Policies (ABPs)
#
# THIS CODE AND INFORMATION IS PROVIDED TO YOU FOR YOUR REFERENTIAL PURPOSES
# ONLY, AND IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EITHER
# EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE, AND MAY NOT
# BE REDISTRIBUTED IN ANY MANNER.
#
# Copyright (C) 2012 Microsoft Corporation.  All rights reserved
#--------------------------------------------------------------------------------

### Modify these variables with values appropriate to your environment ###

$PreferredDC = "DC01.sp2enterprise.net"
$DomainID = "sp2enterprise"
$DomainSuffix = "net"
$HostingID = "Hosting"
$ResellerID = "Reseller001"

$BaseADPath = "OU=$ResellerID,OU=$HostingID,DC=$DomainID,DC=$DomainSuffix"

################### You should not need to edit any variables below this line ###################

if ($args.length -ne 1)
{
	Write-Host "You must provide the name of the customer OU as an argument." -foregroundcolor yellow
	Write-Host "if there are spaces or special characters in the name, surround it with quotes." -foregroundcolor yellow
	Write-Host "Exiting script." -foregroundcolor yellow
	exit
}
$TenantID = $args[0]

## Open error logfile
$Timestamp = get-date -f yyyy-mm-dd-hhmmss
$Logfile = "StampCustomAttributeLog-" + "$TenantID-" + $Timestamp + ".txt"
get-date | out-file $Logfile
"Beginning Bulk Stamping of Custom Attribute on all Customer Organization mail-enabled objects..." | out-file $Logfile -append
"-------------------------------------------------" | out-file $Logfile -append


# create a dynamic array to hold mail-enabled object DNs
$arrMailObjectDNs = @()

$MailObjectsFound = 0
$MailObjectsUpdated = 0
$MailObjectsFailed = 0

##########################################################################
# do a onelevel search on the tenant OU and retrieve all object DNs

Write-Host "Searching the tenant OU for mail-enabled users,contacts, and groups..." -foregroundcolor cyan

$strFilter = "(&(|(objectClass=user)(objectClass=contact)(objectClass=group)))"
$objDomain = New-Object System.DirectoryServices.DirectoryEntry("LDAP://$PreferredDC/OU=$TenantID,$BaseADPath")

$objSearcher = New-Object System.DirectoryServices.DirectorySearcher
$objSearcher.SearchRoot = $objDomain
$objSearcher.PageSize = 1000
$objSearcher.Filter = $strFilter
$objSearcher.SearchScope = "Onelevel" #search only the OU, not sub-OUs

#list of attributes to retrieve values for.  if you remark out the next two lines, all properties will be retrieved.
$colProplist = "distinguishedName","mailNickname"
foreach ($i in $colPropList){$objSearcher.PropertiesToLoad.Add($i)}

$colResults = $objSearcher.FindAll()

foreach ($objResult in $colResults)
{
	$objItem = $objResult.Properties
	# list each property you want to retrieve.  attribute must be specified in all lower case!
	[string]$MailObjectDN = $objItem.distinguishedname
	[string]$MailObjectAlias = $objItem.mailnickname

	if ($MailObjectAlias -gt "") # we are only interested in mail-enabled users, contacts, and groups
	{
		$MailObjectsFound += 1
		Write-Host "found mail-enabled object $MailObjectAlias" -foregroundcolor green
		"found mail-enabled object $MailObjectAlias" | out-file $Logfile -append
		$arrMailObjectDNs += $MailObjectDN
	}
	else
	{
		# the object is not mail-enabled, we're not interested in it.
		#write-host "object $MailObjectDN is not mail-enabled... skipping." -foregroundcolor yellow
		"object $MailObjectDN is not mail-enabled... skipping." | out-file $Logfile -append
	}
}
##########################################################################

# stamp the tenant name onto CustomAttribute1 of each mail-enabled object in the tenant OU
foreach ($MailObject in $arrMailObjectDNs)
{ 
	## Update the mail-enabled object
	Write-Host "Setting CustomAttribute1 for mail-enabled object $MailObject." -foregroundcolor cyan		

	#Clear Error Count
	$error.clear()
	$oldErrCount = $error.Count

	#Bind to the UserObject
	$objMailObject = [ADSI]"LDAP://$PreferredDC/$MailObject"

	$objMailObject.put("extensionAttribute1","$TenantID")
	$objMailObject.setInfo()

	if ($error.Count -ne $oldErrCount)
	{
		[string]$ErrorInfo = $Error[$error.count-1].exception
		$MailObjectsFailed += 1
	}
	else
	{
		$MailObjectsUpdated += 1
		# Update the mail-enabled object so it will appear in the right address lists ASAP
		Update-Recipient $MailObject -DomainController $PreferredDC
	}
}

### Logging and Reporting
Write-Host "--------------------------------------------"
Write-Host "Mail-enabled objects found in tenant OU: $MailObjectsFound"
Write-Host "Mail-enabled objects successfully prepared: $MailObjectsUpdated"
Write-Host "Mail-enabled objects failed to be prepared: $MailObjectsFailed"
Write-Host "Operation complete." -foregroundcolor cyan


"--------------------------------------------" | out-file $Logfile -append
"Mail-enabled objects found in tenant OU: $MailObjectsFound" | out-file $Logfile -append
"Mail-enabled objects successfully prepared: $MailObjectsUpdated" | out-file $Logfile -append
"Mail-enabled objects failed to be prepared: $MailObjectsFailed" | out-file $Logfile -append
"Operation complete." | out-file $Logfile -append
